;******************************************************************************
;   This file is a basic code template for code generation on the             *
;   PIC16F19155. This file contains the basic code building blocks to build     *
;   upon.                                                                     *
;                                                                             *
;   Refer to the MPASM User's Guide for additional information on             *
;   features of the assembler.                                                *
;                                                                             *
;   Refer to the respective data sheet for additional                         *
;   information on the instruction set.                                       *
;                                                                             *
;******************************************************************************
;                                                                             *
;    Filename:        KELVIN METER.asm                                        *
;    Date:           SEPTEMBER 29, 2019                                       *
;    File Version:     1                                                      *
;    DATE:  10/28/2019
;    REV 2  FINAL									      *
;    Author:       Ron Newton                                                 *
;    Company:                                                                 *
;                                                                             *
;                                                                             *
;******************************************************************************
;                                                                             *
;    Files Required: PIC16F1955.INC                                           *
;                                                                             *
;******************************************************************************
;                                                                             *
;    Notes:                                                                   *
;                                                                             *
;******************************************************************************
;							                    *
;         
;	                          					    *
;       
;	                                                                    *
;******************************************************************************


	list		p=16F19155      ; list directive to define processor
	#include	<p16F19155.inc> ; processor specific variable definitions

;------------------------------------------------------------------------------
;
; CONFIGURATION WORD SETUP
;
; The 'CONFIG' directive is used to embed the configuration word within the 
; .asm file. The lables following the directive are located in the respective 
; .inc file.  See the data sheet for additional information on configuration 
; word settings.
;
    __CONFIG _CONFIG1, _FEXTOSC_OFF & _RSTOSC_HFINT32 & _CLKOUTEN_ON & _LCDPEN_OFF & _VBATEN_OFF & _CSWEN_ON & _FCMEN_ON 
    __CONFIG _CONFIG2, _MCLRE_ON & _PWRTE_OFF & _LPBOREN_OFF & _BOREN_ON & _BORV_LO & _ZCD_OFF & _PPS1WAY_ON & _STVREN_ON
    __CONFIG _CONFIG3, _WDTCPS_WDTCPS_31 & _WDTE_ON & _WDTCWS_WDTCWS_7 & _WDTCCS_LFINTOSC
    __CONFIG _CONFIG4, _BBSIZE_512 & _BBEN_OFF & _SAFEN_OFF
;------------------------------------------------------------------------------    
;1	MCLR		NC	    MCLR
;2	RA0		RELAY	
;3	RA1		POWER
;4	RA2		PROBE IN
;5	RA3		NC
;6	RA4		NC
;7	RA5		NC
;8	GND
;9	RA7		NC
;10	RA6		NC
;11	RC0		SW2 UP
;12	RC1		SW3 DOWN
;13	RC2		FUTURE
;14	RC3		S4
;15	RC4		RS
;16	RC5		NC
;17	RC6		R/W
;18	RC7		E
;19	GND
;20	V POS
;21	RB0		D7
;22	RB1		D6
;23	RB2		D5
;24	RB3		D4
;25	RB4		D3
;26	RB5		D2
;27	RB6		D1	ICSPCLK
;28	RB7		D0	ICSPDAT



;------------------------------------------------------------------------------
; VARIABLE DEFINITIONS
;
; Available Data Memory divided into Bank 0,1,2, and 3.  Each Bank contains 
; Special Function Registers, General Purpose Registers, and Access RAM 
;
;------------------------------------------------------------------------------

;       Bit Equates 
;****************************************************************************
;DONE    equ     7               ;done measuring flag
EMPTY   equ     5               ;flag if component is connected
V1      equ     5               ;power for C reference ckt
msb_bit equ     7               ;define for bit 7
lsb_bit equ     0               ;define for bit 0
time	equ		3
sign    equ		4
DI			equ		7							;I2C PROGRAMS
DO			equ		6
SDA			equ		2
SCL			equ		1
ERHUNDREDS	equ		7

    CBLOCK 0x20 ; Define GPR variable register locations


;****************************************************************************
;       User Registers
;****************************************************************************
;       Bank 0
;NEW REGISTERS

COUNT3	
COUNT4	
COUNT5	
msb    	            ;general delay register
lsb    	            ;general delay register
TIMER1
TIMER2
TIMER3
FLAG
FLAGS
FLAG1
FLAG2
ADJUSTMENT 

;		DOG REGISTERS
DATA1
COUNTER0
COUNTER1
ONES
TENS
HUNDREDS
THOUSANDS
DIGIT_1
DIGIT_2
DIGIT_3
DIGIT_4
DIGIT_5
DIVISOR_L
DIVISOR_M
FACTOR    
REVERSE    



;       Math related registers
ACCaHI  	           ;24-Bit accumulator a
ACCaMID 	
ACCaLO  	
ACCbHI  	           ;24-Bit accumulator b
ACCbMID 	
ACCbLO  	
ACCcHI  	           ;24-Bit accumulator c
ACCcMID 	
ACCcLO  	
ACCdHI  	           ;24-Bit accumulator d
ACCdMID 	
ACCdLO  	
TENK					;10,000
temp   	           		;temporary storage
    ENDC

W_TEMP         equ    0x7D        ; w register for context saving (ACCESS)
STATUS_TEMP    equ    0x7E        ; status used for context saving (ACCESS)
PCLATH_TEMP    equ    0x7F        ; context saving of PC (ACCESS) 
#DEFINE		RW				PORTC,6		;RW OUT
#DEFINE		RS				PORTC,4		;RS OUT
#DEFINE		E				PORTC,7		;E OUT
#DEFINE		UP				PORTC,0
#DEFINE		DOWN				PORTC,1
#DEFINE		POWER				PORTA,1
#DEFINE		RELAY				PORTA,0
#DEFINE		SWITCH_4			PORTC,2	
   
;------------------------------------------------------------------------------
; EEPROM INITIALIZATION
;
; 
;------------------------------------------------------------------------------




;------------------------------------------------------------------------------
; RESET VECTOR
;------------------------------------------------------------------------------

    ORG     0x0000            ; processor reset vector
	BTFSS	STATUS,4
	GOTO	OFF
	PAGESEL init
        goto    init

;------------------------------------------------------------------------------
; INTERRUPT SERVICE ROUTINE
;------------------------------------------------------------------------------


	org 4	

    ; Interrupt context saving

    

;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
INT2
    ; Interrupt context restoring

    
;------------------------------------------------------------------------------
; MAIN PROGRAM
;------------------------------------------------------------------------------

init
        BANKSEL	OSCCON1
	MOVLW	B'00000011'	    ;4 MHZ
	MOVWF	OSCCON1
	BANKSEL	PCON0
	BTFSS	PCON0,4		    ;CHECK FOR WDT RESET
	BCF	POWER		    ;TURN OFF POWER
	BANKSEL	PORTA			;change banks
        CLRF    PORTA		       ;clear PORTA, PORTB and PORTC
	BANKSEL	LATA
	CLRF	LATA
	BANKSEL	ANSELA
	MOVLW	B'00000100'		;MAKE RA2 ANALOG
	MOVWF	ANSELA
	BANKSEL	TRISA
	MOVLW	B'00000100'		;MAKE RA2 INPUT
	MOVWF	TRISA
	BANKSEL	PORTB
	CLRF    PORTB
	BANKSEL	LATB
	CLRF	LATB
	BANKSEL	ANSELB
	CLRF	ANSELB
	BANKSEL TRISB
	CLRF	TRISB			;MAKE PORTB ALL OUTPUT
	BANKSEL	PORTC
	CLRF	PORTC
	BANKSEL	LATC
	CLRF	LATC
	MOVLW	B'00001111'		;MAKE PORTC 0-3 INPUTS
	MOVWF	TRISC
	BANKSEL	WPUC
	MOVLW	B'00001111'		;PULLUP ON PORTC 0 - 3
	MOVWF	WPUC
	BANKSEL	ANSELC
	CLRF	ANSELC
	BANKSEL	TRISC	
	BANKSEL FVRCON
	MOVLW	B'10000011'
	MOVWF	FVRCON			;FIX VOLTAGE TO 4.096 V
	BANKSEL ADCON0
	MOVLW	B'10010100'		;A/D ON RIGHT JUSTIFIED
	MOVWF	ADCON0
	BANKSEL	ADCLK
	MOVLW	B'00111111'
	MOVWF	ADCLK
	BANKSEL	ADREF
	MOVLW	B'00000011'
	MOVWF	ADREF			;USING VFR
	BANKSEL	ADPCH
	MOVLW	0X02
	MOVWF	ADPCH			;RA2
	BANKSEL	WDTCON0
	MOVLW	B'00100011'		;128 SECONDS TIMEOUT
	MOVWF	WDTCON0
	CLRWDT				;CLEAR WATCH DOG
	BANKSEL	PORTA
	CLRF	ADJUSTMENT
	BSF	POWER		;TURN ON POWER		
	BCF	RELAY		;TURN OFF RELAY
;********************TEST AREA*********************


	
;******************************* TESTING ********************************
; DISPLAYS FOR SHORT TIME UNIT NAME AND WHAT PROGRAM IT IS USING
BEGIN 
		CLRF	FLAG1
		CLRF	FLAG2
		BTFSS	SWITCH_4
		GOTO	BA
		BCF	FLAG1,0
		GOTO	BB
BA		BSF	FLAG1,0
BB		BTFSC	SWITCH_4
		GOTO	B1
;THIS SECTION FOR 1 TO .001 OHMS		
			MOVLW	0X89 		;DIVISOR LOW
		MOVWF	DIVISOR_L
		MOVLW	0X1		;DIVISOR = 404
		MOVWF	DIVISOR_M	;DIVISOR MID
		MOVLW	.100		;FACTOR
		MOVWF	FACTOR
		CALL	DOG		;INITIALIZE LCD
		MOVLW	04BH		;KELVIN
		CALL	SEND
		MOVLW	045H		;E
		CALL	SEND
		MOVLW	04CH		;L
		CALL	SEND		
		MOVLW	056H		;V
		CALL	SEND
		MOVLW	049H		;I
		CALL	SEND
		MOVLW	04EH		;N
		CALL	SEND
		MOVLW	20H		;SPACE
		CALL	SEND
		MOVLW	04FH		;OHM
		CALL	SEND	
		MOVLW	48H		;H
		CALL	SEND	
		MOVLW	04DH		;M
		CALL	SEND	
		MOVLW	20H		;SPACE
		CALL	SEND	
		MOVLW	04DH		;METER
		CALL	SEND	
		MOVLW	45H		;E
		CALL	SEND	
		MOVLW	54H		;T
		CALL	SEND	
		MOVLW	45H		;E
		CALL	SEND	
		MOVLW	52H		;R
		CALL	SEND	
		MOVLW	B'10101000'		;CHANGE TO SECOND LINE 40
		MOVWF	DATA1
		CALL	WRINS_CHK		; SECOND LINE	
	
		MOVLW	20H		;SPACE
		CALL	SEND	
		MOVLW	20H		;SPACE
		CALL	SEND	
		MOVLW	31H		;1
		CALL	SEND
		MOVLW	20H		;SPACE
		CALL	SEND			
		MOVLW	01EH		;OHM SIGN
		CALL	SEND	
		MOVLW	20H		;SPACE
		CALL	SEND	
		MOVLW	02DH		;-
		CALL	SEND	
		MOVLW	20H		;SPACE	
		CALL	SEND	
		MOVLW	030H		;0.001
		CALL	SEND	
		MOVLW	02EH		;.
		CALL	SEND	
		MOVLW	030H		;0
		CALL	SEND	
		MOVLW	030H		;0
		CALL	SEND	
		MOVLW	31H		;1
		CALL	SEND	
		MOVLW	20H		;SPACE
		CALL	SEND	
		MOVLW	01EH		;OHM SIGN
		CALL	SEND	
		CALL	DELAY2
		GOTO	START
;THIS SECTION 40 TO .01
B1			MOVLW	.101		;DIVISOR LOW INCREASE FROM .100
		MOVWF	DIVISOR_L
		MOVLW	0X0		;DIVISOR = 98
		MOVWF	DIVISOR_M	;DIVISOR MID
		MOVLW	.100		;FACTOR
		MOVWF	FACTOR
		CALL	DOG		;INITIALIZE LCD
		MOVLW	04BH		;KELVIN
		CALL	SEND
		MOVLW	045H		;E
		CALL	SEND
		MOVLW	04CH		;L
		CALL	SEND		
		MOVLW	056H		;V
		CALL	SEND
		MOVLW	0049H		;I
		CALL	SEND
		MOVLW	04EH		;N
		CALL	SEND

		MOVLW	20H		;SPACE
		CALL	SEND
		MOVLW	04FH		;OHM
		CALL	SEND	
		MOVLW	48H		;H
		CALL	SEND	
		MOVLW	04DH		;M
		CALL	SEND	
		MOVLW	20H		;SPACE
		CALL	SEND	
		MOVLW	04DH		;METER
		CALL	SEND	
		MOVLW	45H		;E
		CALL	SEND	
		MOVLW	54H		;T
		CALL	SEND	
		MOVLW	45H		;E
		CALL	SEND	
		MOVLW	52H		;R
		CALL	SEND	
		MOVLW	31H		;1
		CALL	SEND	
		MOVLW	030H		;0
		CALL	SEND	
		MOVLW	030H		;0
		CALL	SEND	
		MOVLW	B'10101000'		;CHANGE TO SECOND LINE 40
		MOVWF	DATA1
		CALL	WRINS_CHK		; SECOND LINE	
		MOVLW	20H
		CALL	SEND
		MOVLW	20H
		CALL	SEND
		MOVLW	33H	    ;38
		CALL	SEND
		MOVLW	38H	    ;8
		CALL	SEND
		MOVLW	20H		;SPACE
		CALL	SEND	
		MOVLW	01EH		;OHM SIGN
		CALL	SEND	
		MOVLW	20H		;SPACE
		CALL	SEND	
		MOVLW	02DH		;-
		CALL	SEND	
		MOVLW	20H		;SPACE	
		CALL	SEND	
		MOVLW	030H		;0.01
		CALL	SEND	
		MOVLW	02EH		;.
		CALL	SEND	
		MOVLW	30H		;0
		CALL	SEND
		MOVLW	31H		;1
		CALL	SEND	
		MOVLW	20H		;SPACE
		CALL	SEND	
		MOVLW	01EH		;OHM SIGN
		CALL	SEND	
		CALL	DELAY2
		GOTO	START



;****************DELAY FOR DISPLAYING
DELAY2		MOVLW		.25
		MOVWF		COUNT5
		DECFSZ		COUNT3,1	;COUNTER
		GOTO		$-1		;CONTINUE
		DECFSZ		COUNT4,1
		GOTO		$-3
		DECFSZ		COUNT5,1
		GOTO		$-5
		RETURN



;*******************************STARTING UP ************************

START		;CHECK SWITCH-4 FOR CHANGE
		BTFSS	SWITCH_4
		GOTO	S1
		BSF	FLAG2,0
		GOTO	S2
S1		BCF	FLAG2,0
S2		MOVF	FLAG1,0
		SUBWF	FLAG2,0
		BTFSC	3,2
		GOTO	BEGIN
;-------------------GETTING CALIBRATION ----------------
SS
		
		BANKSEL	NVMADRL
		MOVLW	0X70
		MOVWF	NVMADRH
		MOVLW	.0
		MOVWF	NVMADRL
		BSF	NVMCON1,NVMREGS	
		BSF	NVMCON1,RD
		MOVF	NVMDATL,W
		BANKSEL	PORTC
		MOVWF	ADJUSTMENT
		BANKSEL	NVMADRL
		CLRF	NVMCON1
		BANKSEL	PORTA
; CHECKING FOR CALIBRATION SWITCHES BEING PUSHED	
	
		BANKSEL	PORTA
		NOP
		BTFSS	DOWN
		GOTO	ADJUST_DOWN
		BTFSS	UP
		GOTO	ADJUST_UP
S5		BSF	RELAY		    ;TURN ON RELAY
		DECFSZ   COUNTER0,1
		GOTO	S5
		BANKSEL	ADCON0		   ;MEASURING VOLTAGE
		BSF	ADCON0,GO
		BTFSC	ADCON0,GO
		GOTO	$-1
		BANKSEL PORTA
		BCF	RELAY		    ;TURN OFF RELAY
		CALL	CALCULATION	    ;START CALCULATION
		MOVLW	.2
		MOVWF	COUNT5
S6	
		BANKSEL	PORTA
		BCF	RELAY 
		DECFSZ	COUNT3,1	    ;DISPLAY DELAY
		GOTO	S6
		DECFSZ	COUNT4,1
		GOTO	S6
		DECFSZ	COUNT5,1
		GOTO	S6
		GOTO	START		    ;DO OVER

		
ADJUST_DOWN
		
		CALL	DOG
		MOVLW	43H	    ;CALIBRATION DOWN
		CALL	SEND
		MOVLW	41H	    ;A
		CALL	SEND
		MOVLW	04CH	    ;L
		CALL	SEND
		MOVLW	49H	    ;I
		CALL	SEND
		MOVLW	42H	    ;B
		CALL	SEND
		MOVLW	52H	    ;R
		CALL	SEND
		MOVLW	41H	    ;A
		CALL	SEND
		MOVLW	54H	    ;T
		CALL	SEND
		MOVLW	49H	    ;I
		CALL	SEND	    
		MOVLW	04FH	    ;O
		CALL	SEND	    
		MOVLW	04EH	    ;N
		CALL	SEND
		MOVLW	20H	    ;SPACE
		CALL	SEND
		MOVLW	44H	    ;D
		CALL	SEND
		MOVLW	04FH	    ;O
		CALL	SEND
		MOVLW	57H	    ;W
		CALL	SEND	  
		MOVLW	04EH	    ;N
		CALL	SEND	 
		BTFSS	DOWN	    ;WAIT UNTIL RELEASED
		GOTO	$-1
		DECF	ADJUSTMENT,1
		MOVF	ADJUSTMENT,W
		CALL	WRITE_EEPROM
		GOTO	START
;----------------WRITING TO  EEPROM----------

WRITE_EEPROM	MOVLW	.10
		MOVWF	COUNTER1
HERE	
		BANKSEL	NVMADRL
		CLRF	NVMCON1
		BSF	NVMCON1,NVMREGS
		BSF	NVMCON1,WREN
		MOVLW	0
		MOVWF	NVMADRL
		MOVLW	0X70
		MOVWF	NVMADRH
		BSF	NVMCON1,NVMREGS
		BSF	NVMCON1,WREN
		BSF	NVMCON1,LWLO
		BANKSEL	PORTA
		MOVF	ADJUSTMENT,0
		BANKSEL	NVMADRL
		MOVWF	NVMDATL
		CLRF	NVMDATH
		BCF	NVMCON1,LWLO
		MOVLW	0X55
		MOVWF	NVMCON2
		MOVLW	0XAA
		MOVWF	NVMCON2
		BSF	NVMCON1,WR
		BTFSC	NVMCON1,WR
		GOTO	$-1
		BCF	NVMCON1,WREN
		CLRF	NVMCON1
		BANKSEL	PORTA
		DECFSZ	COUNTER1,1
		GOTO	HERE
		BANKSEL	PORTA
		RETURN
		
ADJUST_UP
		
		CALL	DOG
		MOVLW	20H
		CALL	SEND
		MOVLW	43H	    ;CALIBRATION UP
		CALL	SEND
		MOVLW	41H	    ;A
		CALL	SEND
		MOVLW	04CH	    ;L
		CALL	SEND
		MOVLW	49H	    ;I
		CALL	SEND
		MOVLW	42H	    ;B
		CALL	SEND
		MOVLW	52H	    ;R
		CALL	SEND
		MOVLW	41H	    ;A
		CALL	SEND
		MOVLW	54H	    ;T
		CALL	SEND
		MOVLW	49H	    ;I
		CALL	SEND	    
		MOVLW	04FH	    ;O
		CALL	SEND	    
		MOVLW	04EH	    ;N
		CALL	SEND
		MOVLW	20H	    ;SPACE
		CALL	SEND
		MOVLW	55H	    ;U
		CALL	SEND
		MOVLW	050H	    ;P
		CALL	SEND
		BTFSS	UP	    ;WAIT UNTIL RELEASED
		GOTO	$-1
		INCF	ADJUSTMENT,1
		MOVF	ADJUSTMENT,0
		CALL	WRITE_EEPROM
		GOTO	START

		
;**************************CALCULATION***********************

	
CALCULATION
		BANKSEL	PORTA
		CALL	CLEAR1	     ;CLEARS ALL MULTIPLICTIONS REGISTORS
		MOVF	FACTOR,W	    ;MULTIPLY BY FACTOR
		MOVWF	ACCaLO
		BANKSEL	ADRESL
		MOVF	ADRESL,W
		BANKSEL	PORTA
;TEST POINT
;	MOVLW	.105	    
		MOVWF	ACCbLO
		MOVF	ADJUSTMENT,0
		ADDWF	ACCbLO,1
		BANKSEL	ADRESH
		MOVF	ADRESH,W
		BANKSEL	PORTA
;TEST POINT
;	MOVLW	0	
		MOVWF	ACCbMID
		CALL	Mpy24
			
		MOVF	DIVISOR_L,W
		MOVWF	ACCaLO
		MOVF	DIVISOR_M,W	;DIVISOR
		MOVWF	ACCaMID
		CALL	Div24
		CALL	DECIMAL
		CALL	DISPLAYING
		BTFSC	SWITCH_4
		GOTO	C1
; 1 OHM		
		MOVLW	0FH
		BANKSEL	ADRESH
		SUBWF	ADRESH,0
		BANKSEL	PORTA
		BTFSC	3,2
		GOTO	C0  
		CALL	DOG
		MOVLW	20H
		CALL	SEND
		MOVLW	20H
		CALL	SEND
		MOVF	DIGIT_4,W
		CALL	SEND
		MOVLW	02EH	    ;DECIMAL
		CALL	SEND			
		MOVF	DIGIT_3,W
		CALL	SEND
		MOVF	DIGIT_2,W
		CALL	SEND

		MOVF	DIGIT_1,W
		CALL	SEND
		MOVLW	20H
		CALL	SEND
		MOVLW	4FH	    ;OHMS
		CALL	SEND
		MOVLW	48H
		CALL	SEND
		MOVLW	4DH
		CALL	SEND
		MOVLW	53H
		CALL	SEND
		RETURN
C0
		BANKSEL	PORTA
		CALL	DOG
		MOVLW	20H
		CALL	SEND
		MOVLW	41H	    ;ABOVE ONE OHM
		CALL	SEND
		MOVLW	42H	    ;B
		CALL	SEND
		MOVLW	4FH	    ;O
		CALL	SEND
		MOVLW	56H	    ;V
		CALL	SEND
		MOVLW	45H	    ;E
		CALL	SEND
		MOVLW	20H
		CALL	SEND
		MOVLW	4FH	    ;O
		CALL	SEND
		MOVLW	4EH	    ;N
		CALL	SEND
		MOVLW	45H	    ;E
		CALL	SEND		
		MOVLW	20H
		CALL	SEND
		MOVLW	4FH	    ;O
		CALL	SEND
		MOVLW	48H	    ;H 
		CALL	SEND
		MOVLW	4DH	    ;M   
		CALL	SEND

		RETURN
		
;  40 OHMS 		
C1		MOVLW	0FH
		BANKSEL	ADRESH
		SUBWF	ADRESH,0
		BANKSEL	PORTA
		BTFSC	3,2
		GOTO	C2
		CALL	DOG
		MOVLW	20H
		CALL	SEND
		MOVLW	20H
		CALL	SEND
		MOVLW	20H
		CALL	SEND
		MOVF	DIGIT_5,W
		CALL	SEND
		MOVF	DIGIT_4,W
		CALL	SEND
		MOVF	DIGIT_3,W
		CALL	SEND
		MOVLW	02EH	    ;DECIMAL
		CALL	SEND			
		MOVF	DIGIT_2,W
		CALL	SEND
		MOVF	DIGIT_1,W
		CALL	SEND
		MOVLW	20H
		CALL	SEND
		MOVLW	4FH	    ;OHMS
		CALL	SEND
		MOVLW	48H
		CALL	SEND
		MOVLW	4DH
		CALL	SEND
		MOVLW	53H
		CALL	SEND
		RETURN

C2	
		BANKSEL	PORTA
		CALL	DOG
		MOVLW	20H
		CALL	SEND
		MOVLW	41H	    ;ABOVE 40 OHMS
		CALL	SEND
		MOVLW	42H	    ;B
		CALL	SEND
		MOVLW	4FH	    ;O
		CALL	SEND
		MOVLW	56H	    ;V
		CALL	SEND
		MOVLW	45H	    ;E
		CALL	SEND
		MOVLW	20H
		CALL	SEND
		MOVLW	34H	    ;4
		CALL	SEND
		MOVLW	30H	    ;0
		CALL	SEND
		MOVLW	20H
		CALL	SEND
		MOVLW	4FH	    ;O
		CALL	SEND
		MOVLW	48H	    ;H
		CALL	SEND
		MOVLW	4DH	    ;M
		CALL	SEND
		MOVLW	53H	    ;S
		CALL	SEND
	
		RETURN
;*******************************************************************;
;****************************CONVERTING TO SEVEN SEG DECIMAL ****************
DECIMAL											;CANNED ROUTINE FROM MICROCHIP NOT DOCUMENTED

		SWAPF		ACCcMID,W
		ANDLW		0FH
		ADDLW		0F0H
		MOVWF		THOUSANDS
		ADDWF		THOUSANDS,F
		ADDLW		0E2H
		MOVWF		HUNDREDS
		ADDLW		32H
		MOVWF		ONES

		MOVF		ACCcMID,0
		ANDLW		0FH
		ADDWF		HUNDREDS,F
		ADDWF		HUNDREDS,F
		ADDWF		ONES,F
		ADDLW		0E9H
		MOVWF		TENS
		ADDWF		TENS,F
		ADDWF		TENS,F

		SWAPF		ACCcLO,W
		ANDLW		0FH
		ADDWF		TENS,F
		ADDWF		ONES,F

		RLF		TENS,F
		RLF		ONES,F
		COMF		ONES,F
		RLF		ONES,F
			
		MOVF		ACCcLO,W
		ANDLW		0FH
		ADDWF		ONES,F
		RLF		THOUSANDS,F
		
		MOVLW		07H
		MOVWF		TENK

		MOVLW		0AH
LB1		ADDWF		ONES,F
		DECF		TENS,F
		BTFSS		3,0
		GOTO		LB1
LB2		ADDWF		TENS,F
		DECF		HUNDREDS,F
		BTFSS		3,0
		GOTO		LB2

LB3		ADDWF		HUNDREDS,F
		DECF		THOUSANDS,F
		BTFSS		3,0
		GOTO		LB3

LB4		ADDWF		THOUSANDS,F
		DECF		TENK,F
		BTFSS		3,0
		GOTO		LB4

		RETURN

;***************************END OF HEXADECIMAL TO DECIMAL ****************************
;****************************************************************************
;*--------------------------------------------------------------------------*
;*-     Division ( 56 / 24 -> 24 )                                         -*
;*-                                                                        -*
;*-     Uses ACCa, ACCb, ACCc, ACCd                                        -*
;*-                                                                        -*
;*-     56-bit dividend in ACCb,ACCc ( ACCb has msb's and ACCc has lsb's)  -*
;*-     24-bit divisor in ACCa                                             -*
;*-     quotient is stored in ACCc                                         -*
;*-     remainder is stored in ACCb                                        -*
;*--------------------------------------------------------------------------*
;****************************************************************************
Div24
        call    Dsetup

dloop   bcf     STATUS,C
        rlf     ACCcLO,1         ;Rotate dividend left 1 bit position
        rlf     ACCcMID,1
        rlf     ACCcHI,1
        rlf     ACCbLO,1
        rlf     ACCbMID,1
        rlf     ACCbHI,1

        btfsc   STATUS,C        ;invert carry and exclusive or with the
        goto    clear           ;msb of the divisor then move this bit
        btfss   ACCaHI,msb_bit  ;into the lsb of the dividend
        incf    ACCcLO,1
        goto    cont
clear   btfsc   ACCaHI,msb_bit
        incf    ACCcLO,1

cont    btfsc   ACCcLO,lsb_bit  ;check the lsb of the dividend
        goto    minus
        call    Add24           ;if = 0, then add divisor to upper 24 bits
        goto    check           ;of dividend
minus   call    Sub24           ;if = 1, then subtract divisor from upper
                                ;24 bits of dividend

check   decfsz  temp,f          ;do 24 times
        goto    dloop

        bcf     STATUS,C
        rlf     ACCcLO,1          ;shift lower 24 bits of dividend 1 bit 
        rlf     ACCcMID,1         ;position left
        rlf     ACCcHI,1
        btfsc   ACCbHI,msb_bit  ;exlusive or the inverse of the msb of the
        goto    w1              ;dividend with the msb of the divisor
        btfss   ACCaHI,msb_bit  ;store in the lsb of the dividend
        incf    ACCcLO,1
        goto    wzd
w1      btfsc   ACCaHI,msb_bit
        incf    ACCcLO,1
wzd     btfss   ACCbHI,msb_bit  ;if the msb of the remainder is set and
        goto    wend
        btfsc   ACCaHI,msb_bit  ;the msb of the divisor is not
        goto    wend
        call    Add24           ;add the divisor to the remainder to correct
                                ;for zero partial remainder

wend    retlw   0               ;quotient in 24 lsb's of dividend
                                ;remainder in 24 msb's of dividend

Dsetup        
        movlw   0x18            ;loop 24 times
        movwf   temp
        
        retlw   0
;_______________________________
;____________________________________________________________________________
;****************************************************************************
;*--------------------------------------------------------------------------*       
;*-     Subtraction ( 24 - 24 -> 24 )                                      -*
;*-                                                                        -*
;*-     Uses ACCa, ACCb, ACCd                                              -*
;*-                                                                        -*
;*-     ACCa -> ACCd,                                                      -*
;*-     2's complement ACCa,                                               -*
;*-     call Add24 ( ACCa + ACCb -> ACCb ),                                -*
;*-     ACCd -> ACCa                                                       -*
;*--------------------------------------------------------------------------*
;****************************************************************************
Sub24
        movf    ACCaHI,W                ;Transfer ACCa to ACCd
        movwf   ACCdHI
        movf    ACCaMID,W
        movwf   ACCdMID
        movf    ACCaLO,W
        movwf   ACCdLO
        call    compA                   ;2's complement ACCa
        call    Add24                   ;Add ACCa to ACCb
        movf    ACCdHI,W                ;Transfer ACCd to ACCa
        movwf   ACCaHI
        movf    ACCdMID,W
        movwf   ACCaMID
        movf    ACCdLO,W
        movwf   ACCaLO
        retlw   0
;____________________________________________________________________________
;****************************************************************************
;*--------------------------------------------------------------------------*
;*-     2's Complement                                                     -*
;*-                                                                        -*
;*-     Uses ACCa                                                          -*
;*-                                                                        -*
;*-     Performs 2's complement conversion on ACCa                         -*
;*--------------------------------------------------------------------------*
;****************************************************************************
compA    
        comf    ACCaLO,1          ;invert all bits in accumulator a
        comf    ACCaMID,1
        comf    ACCaHI,1
        incf    ACCaLO,1          ;add one to accumulator a
        btfsc   STATUS,Z
        incf    ACCaMID,1
        btfsc   STATUS,Z
        incf    ACCaHI,1
        retlw   0
;____________________________________________________________________________

;****************************************************************************
;*--------------------------------------------------------------------------*
;*-     Multiply ( 24 X 24 -> 56 )                                         -*
;*-                                                                        -*
;*-     Uses ACCa, ACCb, ACCc, ACCd                                        -*
;*-                                                                        -*
;*-     ACCa * ACCb -> ACCb,ACCc  56-bit output                            -*
;*-     with ACCb (ACCbHI,ACCbMID,ACCbLO) with 24 msb's and                -*
;*-     ACCc (ACCcHI,ACCcMID,ACCcLO) with 24 lsb's                         -*
;*--------------------------------------------------------------------------*
;****************************************************************************
Mpy24
        call    Msetup
mloop   rrf     ACCdHI,1          ;rotate d right
        rrf     ACCdMID,1
        rrf     ACCdLO,1
        btfsc   STATUS,C        ;need to add?
        call    Add24
        rrf     ACCbHI,1
        rrf     ACCbMID,1
        rrf     ACCbLO,1
        rrf     ACCcHI,1
        rrf     ACCcMID,1
        rrf     ACCcLO,1
        decfsz  temp,1            ;loop until all bits checked
        goto    mloop
        retlw   0

Msetup
        movlw   0x18            ;for 24 bit shifts
        movwf   temp
        movf    ACCbHI,W        ;move ACCb to ACCd
        movwf   ACCdHI
        movf    ACCbMID,W
        movwf   ACCdMID
        movf    ACCbLO,W
        movwf   ACCdLO
        clrf    ACCbHI
        clrf    ACCbMID
        clrf    ACCbLO
        retlw   0
;____________________________________________________________________________	
;****************************************************************************
;*--------------------------------------------------------------------------*
;*-     24-bit Addition                                                    -*
;*-                                                                        -*
;*-     Uses ACCa and ACCb                                                 -*
;*-                                                                        -*
;*-     ACCa + ACCb -> ACCb                                                -*
;*--------------------------------------------------------------------------*
;****************************************************************************
Add24
        movf    ACCaLO,W
        addwf   ACCbLO,1          ;add low bytes
        btfsc   STATUS,C        ;add in carry if necessary
        goto    A2
A1      movf    ACCaMID,W
        addwf   ACCbMID,1         ;add mid bytes
        btfsc   STATUS,C        ;add in carry if necessary
        incf    ACCbHI,1
        movf    ACCaHI,W
        addwf   ACCbHI,1          ;add high bytes
        retlw   0
A2      incf    ACCbMID,1
        btfsc   STATUS,Z
        incf    ACCbHI,1
        goto    A1
;____________________________________________________________________________


;****************************************************************************
;****************************************************************************
;*--------------------------------------------------------------------------*
;*-     Delay Routines                                                     -*
;*--------------------------------------------------------------------------*
;****************************************************************************
vlong   movlw   0xff            ;very long delay, approx 200msec
        movwf   msb
        goto    d1
delay20                         ;20 msec delay
        movlw   .26
        movwf   msb
        goto    d1
delay16                         ;16 msec delay
        movlw   .21
        movwf   msb
        goto    d1
delay1                          ;approx 750nsec delay
        movlw   .1      
        movwf   msb
d1      movlw   0xff
        movwf   lsb
d2      decfsz  lsb,F
        goto    d2
        decfsz  msb,F
        goto    d1
        return
;____________________________________________________________________________

;****************************************************************************


;------------------------------------------------------------------------------

SEND		MOVWF		DATA1
		CALL		WRDATA_CHK
		CALL		DELAY30uS
		RETURN
WRDATA_CHK
		CALL		CHK_BUSY
		BSF			RS
		BCF			RW
		BSF			E
		MOVF		DATA1,0
		MOVWF		PORTB
		CALL	    REVERSING
		BCF			E
		MOVLW		.255
		MOVWF		PORTB
		RETURN
WRINS_CHK
		CALL		CHK_BUSY
WRINS_NOCHK
		BCF			RS
		BCF			RW
		BSF			E
		MOVF		DATA1,0
		MOVWF		PORTB
		CALL	    REVERSING
		BCF			E
		MOVLW		.255
		MOVWF		PORTB
		RETURN
CHK_BUSY
		BANKSEL		TRISB		;CHANGING OUTPUT TO INPUT
		MOVLW		.255
		MOVWF		TRISB		;LOAD
		BANKSEL		PORTB
		BCF			RS			;RS LOW
		BSF			RW			;RW HIGH
		BSF			E			;ENABLE HIGH
		CALL		DELAY30uS	;ADDED FOR BUSY CHECK
;		BTFSC		PORTC,7		;CHECK BUSY DOES NOT WORK
;		GOTO		$-1
		BANKSEL		TRISB	
		CLRF		TRISB		;LOAD MAKE OUTPUT
		BANKSEL		PORTA		;CHANGE BANKS
		BCF			E			;LOWER ENABLE
		RETURN					;RETURN


DELAY40MS
		MOVLW		.56
		MOVWF		COUNTER1
		DECFSZ		COUNTER0,1
		GOTO		$-1
		DECFSZ		COUNTER1,1
		GOTO		$-3
		RETURN
DELAY30uS
		MOVLW		.11
		MOVWF		COUNTER0
		DECFSZ		COUNTER0,1
		GOTO		$-1
		RETURN
DELAY2MS
		MOVLW		3
		MOVWF		COUNTER1
		DECFSZ		COUNTER0,1
		GOTO		$-1
		DECFSZ		COUNTER1,1
		GOTO		$-3
		RETURN

;***********************************INTIALIZING LCD DISPLAY*******************
DOG		CALL		DELAY40MS
		MOVLW		039H			;FUNCTION SET INSTRUCTION SET 0
		MOVWF		DATA1
		CALL		WRINS_NOCHK
		CALL		DELAY30uS
		MOVLW		01CH			;SET BIAS 5.0 VOLT
		MOVWF		DATA1
		CALL		WRINS_CHK
		CALL		DELAY30uS
		MOVLW		074H			;CONTRAST SET ADJUSTMENT 5.0 VOLT	
		MOVWF		DATA1
		CALL		WRINS_CHK
		CALL		DELAY30uS
		MOVLW		052H			;POWER/ICON/CONTRAST 5.0 VOLT
		MOVWF		DATA1
		CALL		WRINS_CHK
		CALL		DELAY30uS
		MOVLW		069H			;FOLLOWER CONTROL
		MOVWF		DATA1
		CALL		WRINS_CHK
		CALL		DELAY30uS
		MOVLW		038H			;FUNCTION SET  INSTRUCTION SET 1
		MOVWF		DATA1
		CALL		WRINS_NOCHK
		CALL		DELAY30uS
		MOVLW		00CH			;DISPLAY ON CURSOR OFF
		MOVWF		DATA1
		CALL		WRINS_CHK
		CALL		DELAY30uS
		MOVLW		001H			;CLEAR DISPLAY
		MOVWF		DATA1
		CALL		WRINS_CHK
		CALL		DELAY2MS
		MOVLW		002H			;GO HOME
		MOVWF		DATA1
		CALL		WRINS_CHK
		CALL		DELAY2MS
		MOVLW		006H			;ENTRY MODE SET
		MOVWF		DATA1
		CALL		WRINS_CHK
		CALL		DELAY30uS	
		RETURN


;**************************** DISPLAYING *****************************8

DISPLAYING	
		MOVLW	.0			;LOAD INTO 0 THE LOW ADDRESS OF LOOKUP TABLE LOOKUP TABLE ADDRESS
		ADDWF	ONES,0			;ADD THE OFFSET TO THE LOW ADDRESS (ADRES)
		MOVLW	.5			;LOAD 3 INTO HIGH ADDRESS OF LOOKUP TABLE
		BTFSC	STATUS,C		;CHECK TO SEE IF THE OFFSET ADDED TO LOW LOOKUP TABLE WENT ABOVE 255
		ADDLW	.1			;IF IT DID ADD 1 TO HIGH TABLE BYTE (PCLATH)
		MOVWF	PCLATH
        	MOVF 	ONES,W			 ;MOVE CONTROL INTO W
        	CALL    DISPLAY			 ;GO TO LOOK UP TABLE       
		MOVWF	DIGIT_1
		CLRF	PCLATH

		MOVLW		.0
		ADDWF		TENS,0
		MOVLW		.5
		BTFSC		STATUS,C
		ADDLW		.1
		MOVWF		PCLATH
		MOVF		TENS,0
		CALL		DISPLAY
		MOVWF		DIGIT_2
		CLRF		PCLATH

		MOVLW		.0
		ADDWF		HUNDREDS,0
		MOVLW		.5
		BTFSC		STATUS,C
		ADDLW		.1
		MOVWF		PCLATH
		MOVF		HUNDREDS,0
		CALL		DISPLAY
		MOVWF		DIGIT_3
		
		MOVLW		.0
		ADDWF		THOUSANDS,0
		MOVLW		.5
		BTFSC		STATUS,C
		ADDLW		.1
		MOVWF		PCLATH
		MOVF		THOUSANDS,0
		CALL		DISPLAY
		MOVWF		DIGIT_4
		CLRF		PCLATH

		MOVLW		.0
		ADDWF		TENK,0
		MOVLW		.5
		BTFSC		STATUS,C
		ADDLW		.1
		MOVWF		PCLATH
		MOVF		TENK,0
		CALL		DISPLAY
		MOVWF		DIGIT_5
		CLRF		PCLATH
		RETURN

		ORG	500
DISPLAY
		ADDWF	PCL,.1			;ADD W TO PROGRAM COUNTER AND LOOK UP DATA  "." = DECIMAL #
		RETLW	30H			;0
		RETLW	31H			;1
		RETLW	32H			;2
		RETLW	33H			;3
		RETLW	34H			;4
		RETLW	35H			;5
		RETLW	36H			;6
		RETLW	37H    			;7
		RETLW	38H			;8
		RETLW	39H			;9



DISPLAY_DELAY
		MOVLW		.10
		MOVWF		TIMER3
		DECFSZ		TIMER1,1
		GOTO		$-1
		DECFSZ		TIMER2,1
		GOTO		$-3
		DECFSZ		TIMER3,1
		GOTO		$-5

		RETURN

;********************USED FOR TROUBLE SHOOTING******************
CHECK
		BANKSEL		PORTA
		MOVLW		043H		;C
		CALL		SEND
		MOVLW		048H		;H
		CALL		SEND
		MOVLW		045H		;E
		CALL		SEND
		MOVLW		043H		;C
		CALL		SEND
		MOVLW		04BH		;K

				RETURN
;***************CLEARING MATH REGISTERS
CLEAR1
		CLRF	ACCaHI
		CLRF	ACCaMID
		CLRF	ACCaLO
		CLRF	ACCbHI
		CLRF	ACCbMID
		CLRF	ACCbLO
		CLRF	ACCcHI
		CLRF	ACCcMID
		CLRF	ACCcLO
		CLRF	ACCdHI
		CLRF	ACCdMID
		CLRF	ACCdLO		
		RETURN

;THIS AREA TAKES PORTB AND MAKE PORTB,0 PORTB,7 AND SO ON DUE TO BOARD LAYOUT 
REVERSING
	CLRF	REVERSE
	BTFSC	PORTB,0
	BSF	REVERSE,7
	BTFSC	PORTB,1
	BSF	REVERSE,6
	BTFSC	PORTB,2
	BSF	REVERSE,5
	BTFSC	PORTB,3
	BSF	REVERSE,4
	BTFSC	PORTB,4
	BSF	REVERSE,3
	BTFSC	PORTB,5
	BSF	REVERSE,2
	BTFSC	PORTB,6
	BSF	REVERSE,1
	BTFSC	PORTB,7
	BSF	REVERSE,0
	MOVF	REVERSE,0
	MOVWF	PORTB
	RETURN

OFF
	BANKSEL	PORTA
	BCF	POWER
	GOTO	$-1
  END